from __future__ import annotations

import typing
import urllib

import aiohttp

from .wiki import Wiki
from .factory import WikiParser, ImageParser, MiscParser, Suggestion, HttpHandler
from .image import Image

__all__ = ("Client",)


class Client:
    """
    A class representing a `Client`.
    """

    __slots__ = ("_http",)

    def __init__(
        self, *, session: aiohttp.ClientSession = None, user_agent: str = None
    ) -> None:
        """
        Client Constructor.

        Args:

            session: An instance of [aiohttp.ClientSession](https://docs.aiohttp.org/en/stable/client_reference.html#client-session).
            user_agent: Your user agent string you want to use while making requests, defaults to this library's user string.
        """  # noqa E501
        self._http = HttpHandler(session=session, headers={"User-Agent": user_agent})

    async def __aenter__(self) -> Client:
        return self

    async def __aexit__(self, exc_type, exc_value, exc_tb) -> None:
        await self.close()

    async def acquire_wiki(self, name: str, *, trace: bool = False) -> Wiki:
        """
        Tries looking for the wiki page with this name.

        Args:
            name: Name of the wiki page.
            trace: Whether to try looking for the name which matches the name you have passed then make another api call with this new name.

        Returns:
            A [Wiki](./wiki.md) object

        Tip:
            You must do `trace=True` to ensure a valid return if you are not spamming requests since it makes 2 api calls.
        """  # noqa E501
        if trace:
            name = (await self.acquire_suggestion(name))[0].title

        url = (
            "?action=query&rvslots=main&rvlimit=1&formatversion=2&rvprop=content&"
            f"titles={urllib.parse.quote(name)}&prop=revisions&format=json"
        )

        obj = await self._http.async_call(url=url, cls=WikiParser)
        return Wiki(obj)

    async def acquire_images(
        self, title: str, limit: typing.Optional[typing.Union[int, str]] = None
    ) -> typing.Generator[Image, None, None]:
        """
        Tries looking for the images in this wiki page.

        Args:
            title: Title of the page.
            limit: Number of images you want. (Defaults to max amount)

        Returns:
            A list of [Image](./image.md) objects.
        """  # noqa E501
        url = (
            "?action=query&generator=images&"
            f"gimlimit={limit or 'max'}"
            f"&prop=imageinfo&iiprop=url&titles={urllib.parse.quote(title)}&format=json"
        )

        obj = await self._http.async_call(url=url, cls=ImageParser)
        return obj.marshall(self._http)

    async def acquire_suggestion(
        self, title: str, *, limit: typing.Optional[typing.Union[int, str]] = None
    ) -> typing.List[Suggestion]:
        """
        Tries looking for the wiki page and name which matches title.

        Args:
            title: Title of the page.
            limit: Number of suggestions you want.

        Returns:
            A list of `Suggestion` objects. (`Suggestion` has only two attributes `.title` and `.url`)
        """  # noqa E501

        url = (
            f"?action=opensearch&search={urllib.parse.quote(title)}"
            f"&gimlimit={limit or 'max'}&formatversion=2&format=json"
        )

        obj = await self._http.async_call(url=url, cls=MiscParser)
        return obj._suggest()

    async def close(self) -> None:
        """
        Closes the ClientSession.
        """
        await self._http.close
