from __future__ import annotations

import itertools
import re
import typing
import inspect

from .exceptions import ParsingError

__all__ = ("Helpers", "construct_repr")


class Helpers:
    """
    A helper class for Parsers.
    """

    __slots__ = ("_content",)

    def __init__(
        self,
        *,
        content: typing.Optional[typing.Union[str, typing.List[typing.Any]]] = None,
    ) -> None:
        self._content = content

    @property
    def content(self):
        return self._content

    @content.setter
    def content(self, value):
        self._content = value

    def clean_short_desc(self) -> str:
        """
        Cleans out {{nihongo}} stuff.
        """
        if self._content:
            m = itertools.chain(
                *map(
                    lambda x: re.split(r"(\{\{.*?\|)|(\}\})", x, re.IGNORECASE),
                    self._content,
                )
            )
            f = "".join(filter(lambda g: g is not None, m))
            bta = re.sub(r"(?i){{nihongo\|", "", f).replace("}}", "")
            bta1 = re.sub(r"(?i){{wplink\|", "", bta)

            return bta1
        else:
            raise ParsingError("Couldn't parse the content.")

    def clean_bg(self) -> str:
        """
        Cleans out gallery tag from background.
        """
        pat = r"(?i)</gallery>"

        if isinstance(self._content, str):
            splitted = re.split(pat, self._content)
            return splitted[1] if len(splitted) > 0 else splitted[0]

        else:
            raise ParsingError("Couldn't parse the content.")

    def clean_suggest(self) -> typing.List[typing.Tuple[str, str]]:
        """
        Cleans out empty lists and forms a list.
        """
        newcon = []
        if isinstance(self._content, list):
            self._content.pop(0)

        else:
            raise ParsingError("Couldn't parse the content.")

        try:
            if self._content[1][0] == "":
                self._content.pop(1)

        except IndexError:
            return self._content

        else:
            for sugggestions in zip(*self._content):
                newcon.append(sugggestions)
            return newcon


def construct_repr(cls, obj) -> str:
    """
    Forms a repr for obj.
    """
    attrs = [
        attr
        for attr in inspect.getmembers(obj)
        if not inspect.ismethod(attr[1])
        if not attr[0].startswith("_")
    ]
    fmt = ", ".join(
        f"{attr}={repr(f'{value[:100]}...' if len(str(value)) > 100 else value)}"
        for attr, value in attrs
    )
    return f"{cls.__name__}({fmt})"
