from __future__ import annotations

import io
import json
import os

import typing

import aiohttp
import yarl

from ..factory import exceptions

VERSION = "2.0.0"
CONTACT_AT = "https://github.com/mooncell07/Saki"

__all__ = ("HttpHandler",)


class HttpHandler:
    """
    Handles http stuff.
    """

    __slots__ = ("session", "headers", "_BASE")

    def __init__(
        self,
        *,
        session: aiohttp.ClientSession = None,
        headers: typing.Mapping[str, typing.Optional[str]] = {},
    ) -> None:
        self.session = session
        self.headers = headers
        self._BASE = "https://typemoon.fandom.com/api.php"

    @property
    async def create_ses(self):
        self.session = aiohttp.ClientSession()

    @property
    def set_headers(self):
        """
        sets headers for requesting.
        """
        head = self.headers
        if head["User-Agent"] is None:
            head["User-Agent"] = f"python-saki/{VERSION} ({CONTACT_AT})/BOT"
        head.update({"Accept-Encoding": "gzip"})
        return head

    async def async_call(self, url: str, *, cls: typing.Any = None) -> typing.Any:
        """
        Calls the API.
        """
        if not isinstance(self.session, aiohttp.ClientSession):
            await self.create_ses

        async with self.session.get(
            self._BASE + url, headers=self.set_headers  # type: ignore
        ) as resp:
            if 300 > resp.status >= 200:
                try:
                    dict_ = await resp.json()
                except json.JSONDecodeError:
                    raise exceptions.SakiError(
                        "Recieved data is not json serializable."
                    ) from None
                return cls(dict_) if cls else dict_

            else:
                raise exceptions.BadStatusCodeError(
                    f"Recieved a bad Status Code of {resp.status}."
                ) from None

    async def _read(self, url: typing.Union[str, yarl.URL]) -> io.BytesIO:
        """
        Reads bytes of given Image.
        """
        if self.session is None:
            await self.create_ses

        async with self.session.get(
            url, headers=self.set_headers  # type: ignore
        ) as resp:
            if 300 > resp.status >= 200:
                iob = io.BytesIO(await resp.read())
                return iob
            else:
                raise exceptions.BadStatusCodeError(
                    f"Recieved a bad Status Code of {resp.status}."
                )

    async def _save(
        self,
        url,
        fp: typing.Union[
            typing.Union[str, bytes, os.PathLike[str], os.PathLike[bytes]], int
        ],
    ) -> int:
        """
        Saves the given Image.
        """
        data = await self._read(url=url)
        if isinstance(fp, io.IOBase) and fp.writable():
            written = fp.write(data.read1())
            return written
        else:
            with open(fp, "wb") as f:
                return f.write(data.read1())

    @property
    async def close(self):
        """
        Closes the ClientSession and marks self.session as None.
        """
        if self.session is not None:
            await self.session.close()
            self.session = None
