from __future__ import annotations

import typing
from dataclasses import dataclass

import yarl
from wikitextparser import parse, remove_markup

from ..image import Image
from .exceptions import BadContentRecievedError, WikiNotFound, SakiError
from .helpers import Helpers


__all__ = (
    "WikiParser",
    "WikiParserInterface",
    "ImageParser",
    "MiscParser",
    "Suggestion",
)


class WikiParser:
    """
    A class with methods to parse different parts of response.
    """

    __slots__ = ("data", "main_content", "helper")

    def __init__(self, data: typing.Mapping[str, typing.Any], *args, **kwargs) -> None:

        self.data = data["query"]["pages"][0]
        self.main_content = self.data["revisions"][0]["slots"]["main"]["content"]

        self.helper = Helpers(content=None)

    def __new__(cls, data: typing.Mapping[str, typing.Any], *args, **kwargs):
        try:
            data = data["query"]["pages"][0]

            if data.get("missing"):
                raise WikiNotFound(
                    "Given wiki was not found. Try changing letter case."
                ) from None

        except (KeyError, IndexError, TypeError):
            raise SakiError("Couldn't build Character.") from None

        else:
            return super(WikiParser, cls).__new__(cls)

    @property
    def title(self) -> typing.Optional[str]:
        """
        Returns Page title.
        """
        try:
            return self.data["title"]
        except (KeyError, IndexError, TypeError):
            return None

    @property
    def page_url(self) -> yarl.URL:
        """
        Returns an URL for given query.
        """
        if self.title:
            scheme = "https"
            host = "typemoon.fandom.com"
            path = f"/wiki/{self.title}"
            return yarl.URL.build(scheme=scheme, host=host, path=path)
        else:
            raise BadContentRecievedError(
                "Couldn't form an url because page doesn't have a title."
            ) from None

    @property
    def page_id(self) -> typing.Optional[int]:
        """
        Returns Page ID.
        """
        try:
            return self.data["query"]["pages"][0]["pageid"]
        except (KeyError, IndexError, TypeError):
            return None

    @property
    def background(self) -> typing.Optional[str]:  # type: ignore[return]
        """
        Returns Background Section.
        """
        bgparse = parse(self.main_content)

        for i in range(1, 7):
            bg = bgparse.get_sections(include_subsections=False, level=i)

            if bg != [] and (bg[0].title).lower() == "background":
                to_clean = bg[0].contents
                self.helper.content = to_clean
                return remove_markup(self.helper.clean_bg())

    @property
    def identity(self) -> typing.Optional[str]:  # type: ignore[return]
        """
        Returns Identity Section.
        """
        idtparse = parse(self.main_content)
        for i in range(1, 7):
            idt = idtparse.get_sections(include_subsections=False, level=i)
            if idt != [] and (idt[0].title).lower() == "identity":
                return remove_markup(idt[0].contents)

    @property
    def short_desc(self) -> typing.Optional[str]:  # type: ignore[return]
        """
        Finds for the short description in the content. It is mostly above Profile
        section
        and has Nihongo template.
        """
        parlist = []
        par = (
            parse(self.main_content)
            .plain_text(replace_templates=False)
            .split("==Profile==")[0]
            .splitlines()
        )

        for i in par:
            if "nihongo" in i.lower():
                parlist.append(i)

        parlist.pop(0) if len(parlist) > 1 else parlist
        for i in range(1, 3):
            try:
                if par[par.index(parlist[0]) + i] != "":
                    parlist.append(par[par.index(parlist[0]) + i])
            except IndexError:
                ...

            self.helper.content = parlist
            return self.helper.clean_short_desc()


class WikiParserInterface(WikiParser):
    """
    Top most interface between HttpHandler, WikiParser and Wiki.
    """

    __slots__ = ("data",)

    def __init__(self, data: typing.Mapping[str, typing.Any], *args, **kwargs) -> None:
        self.data = data

        super().__init__(data, *args, **kwargs)


class ImageParser:
    """
    A Parser class for extracting images and performing different functions
    with it.
    """

    __slots__ = ("data",)

    def __init__(self, data: typing.Mapping[str, typing.Any], *, http=None) -> None:
        self.data = data

    def _data_pick(self) -> dict:
        """
        Tries picking up the content from newly arrived dict.
        """
        try:
            return self.data["query"]["pages"]
        except KeyError:
            raise SakiError(f"Recieved malformed data -> {self.data}") from None

    def image_parse(self) -> typing.Generator[str, None, None]:
        """
        Yields image urls.
        """
        con = self._data_pick()
        for page in con:
            if con[page].get("imageinfo") is not None:
                img = con[page].get("imageinfo")[0]["url"]
                yield img

    def marshall(self, http) -> typing.Generator[Image, None, None]:
        """
        Marshalls all urls into Image.
        """
        yield from (Image(img, http) for img in self.image_parse())


@dataclass
class Suggestion:
    title: str
    url: str


class MiscParser:
    """
    A Parser class for misc data extraction.
    """

    __slots__ = ("data",)

    def __init__(self, data):
        self.data = data

    def _suggest(self) -> list:
        """
        returns suggestions.
        """
        hpr = Helpers(content=self.data)
        return [Suggestion(sg[0], sg[1]) for sg in hpr.clean_suggest()]
