from __future__ import annotations

from yarl import URL
import io
import typing
import os

from .factory import construct_repr

if typing.TYPE_CHECKING:
    from .factory import HttpHandler


__all__ = ("Image",)


class Image:
    """
    A class representing an `Image`.

    Attributes:
        url (str): URL of the image.
    """

    __slots__ = ("url", "_http")

    def __init__(self, url: str, http: HttpHandler) -> None:
        self.url = url
        self._http = http

    def __str__(self) -> str:
        """
        Returns:
            URL of the image.
        """
        return self.url

    def __repr__(self) -> str:
        """
        Returns:
            Representation of `Image`.
        """
        return construct_repr(self.__class__, self)

    async def read(self) -> io.BytesIO:
        """
        Reads bytes of the image.

        Returns:
            An [io.BytesIO](https://docs.python.org/3/library/io.html#io.BytesIO) object.
        """  # noqa E501
        read_ret = await self._http._read(url=self.url)
        return read_ret

    async def save(
        self,
        fp: typing.Union[
            typing.Union[str, bytes, os.PathLike[str], os.PathLike[bytes]], int
        ],
    ) -> int:
        """
        Saves the image at the given file path.

        Args:
            fp: Path where the file has to be saved.

        Returns:
            The bytes written in int.
        """
        saved = await self._http._save(self.url, fp)
        return saved

    def deconstruct_url(self) -> URL:
        """
        Makes a [yarl.URL](https://yarl.readthedocs.io/en/latest/api.html#yarl.URL) object out of the image url.

        Returns:
            A [yarl.URL](https://yarl.readthedocs.io/en/latest/api.html#yarl.URL) object.
        """  # noqa E501
        url = URL(self.url)
        return url
