from __future__ import annotations

import typing

import yarl

from .factory import WikiParserInterface, construct_repr

__all__ = ("Wiki",)


class Wiki:
    """
    A class representing a `Wiki`.
    """

    __slots__ = ("_parsed",)

    def __init__(self, _parsed: WikiParserInterface) -> None:
        self._parsed = _parsed

    def __int__(self) -> int:
        """
        Returns:
            ID of the wiki or 0 in case it couldn't extract ID.
        """
        return self.extract_page_id or 0

    def __str__(self) -> str:
        """
        Returns:
            Name of the wiki or ... in case it couldn't extract name.
        """
        return self.extract_name or "..."

    def __eq__(self, value) -> bool:
        """
        Returns:
            Result of `x==x`.
        """
        return (
            isinstance(value, self.__class__)
            and value.extract_page_id == self.extract_page_id
        )

    def __repr__(self) -> str:
        """
        Returns:
            Representation of `Wiki` object.
        """
        return construct_repr(self.__class__, self)

    @property
    def extract_name(self) -> typing.Optional[str]:
        """
        Tries extracting name of the page.

        Returns:
            A string name of the wiki.
        """
        return self._parsed.title

    @property
    def extract_page_url(self) -> typing.Optional[yarl.URL]:
        """
        Tries extracting url of the wiki page.

        Returns:
            A [yarl.URL](https://yarl.readthedocs.io/en/latest/api.html#yarl.URL) object.
        """  # noqa E501
        return self._parsed.page_url

    @property
    def extract_page_id(self) -> typing.Optional[int]:
        """
        Tries extracting id of the page.

        Returns:
            An integer id of the wiki page.
        """
        return self._parsed.page_id

    @property
    def extract_background(self) -> typing.Optional[str]:
        """
        Tries extracting background section, mainly found in character pages like [this](https://typemoon.fandom.com/wiki/Shirou_Emiya).

        Returns:
            A string of character's background.
        """  # noqa E501
        return self._parsed.background

    @property
    def extract_identity(self) -> typing.Optional[str]:
        """
        Tries extracting identity section from the wiki, these are mainly found in character pages too.

        Returns:
            A string of character's identity.
        """  # noqa E501
        return self._parsed.identity

    @property
    def extract_short_desc(self) -> typing.Optional[str]:
        """
        Tries extracting short description from the wiki, these mainly found on top and have around 5-10 lines.

        Returns:
            A string of short description.
        """  # noqa E501
        return self._parsed.short_desc
